"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamicBuffer = void 0;
const util_1 = __importDefault(require("util"));
class DynamicBuffer {
    constructor(buffer) {
        this.buffer = buffer;
        this.cursor = 0;
    }
    static alloc(size, encoding = "utf8") {
        return new DynamicBuffer(Buffer.alloc(Math.max(0, size), null, encoding));
    }
    static from(buf, encoding = "hex") {
        return new DynamicBuffer(Buffer.from(buf, encoding));
    }
    [Symbol.toStringTag]() {
        return "<DynamicBuffer " + this.buffer.toString("hex") + ">";
    }
    [util_1.default.inspect.custom]() {
        return this.buffer.toString();
    }
    *[Symbol.iterator]() {
        let cursor = 0;
        while (cursor < this.buffer.byteLength) {
            yield this.buffer.readUInt8(cursor);
            cursor++;
        }
    }
    get size() {
        return this.buffer.byteLength;
    }
    get left() {
        return this.size - this.cursor;
    }
    toString(encoding = "hex") {
        return this.buffer.toString(encoding);
    }
    goto(pos) {
        this.cursor = pos;
        return this;
    }
    jump(bytes) {
        this.cursor += bytes;
        return this;
    }
    realloc(size) {
        if (size === this.size) {
            return this;
        }
        const newbuf = Buffer.alloc(size);
        const copysz = Math.max(newbuf.byteLength, this.size);
        this.buffer.copy(newbuf, 0, 0, copysz);
        this.buffer = newbuf;
        return this;
    }
    expand(bytes) {
        if (this.cursor + bytes > this.size) {
            this.realloc(this.cursor + bytes);
        }
        return this;
    }
    uint8(val) {
        if (typeof val === "number") {
            this.expand(1);
            this.buffer.writeUInt8(val < 0 ? val >>> 0 : val, this.cursor);
            this.cursor += 1;
            return this;
        }
        else {
            const val = this.buffer.readUInt8(this.cursor);
            this.cursor += 1;
            return val;
        }
    }
    int8(val) {
        if (typeof val === "number") {
            this.expand(1);
            this.buffer.writeInt8(val, this.cursor);
            this.cursor += 1;
            return this;
        }
        else {
            const val = this.buffer.readInt8(this.cursor);
            this.cursor += 1;
            return val;
        }
    }
    byte(val) {
        return this.uint8(val);
    }
    char(val) {
        if (typeof val === "string") {
            return this.uint8(val.charCodeAt(0));
        }
        else {
            return String.fromCharCode(this.uint8());
        }
    }
    bool(val) {
        if (typeof val === "boolean") {
            return this.uint8(val ? 0x01 : 0x00);
        }
        else {
            return this.uint8() === 0x01;
        }
    }
    uint16LE(val) {
        if (typeof val === "number") {
            this.expand(2);
            this.buffer.writeUInt16LE(val < 0 ? val >>> 0 : val, this.cursor);
            this.cursor += 2;
            return this;
        }
        else {
            const val = this.buffer.readUInt16LE(this.cursor);
            this.cursor += 2;
            return val;
        }
    }
    uint16BE(val) {
        if (typeof val === "number") {
            this.expand(2);
            this.buffer.writeUInt16BE(val < 0 ? val >>> 0 : val, this.cursor);
            this.cursor += 2;
            return this;
        }
        else {
            const val = this.buffer.readUInt16BE(this.cursor);
            this.cursor += 2;
            return val;
        }
    }
    uint16(val, endian = "le") {
        return endian === "be" || val === "be" ? this.uint16BE(val) : this.uint16LE(val);
    }
    int16LE(val) {
        if (typeof val === "number") {
            this.expand(2);
            this.buffer.writeInt16LE(val, this.cursor);
            this.cursor += 2;
            return this;
        }
        else {
            const val = this.buffer.readInt16LE(this.cursor);
            this.cursor += 2;
            return val;
        }
    }
    int16BE(val) {
        if (typeof val === "number") {
            this.expand(2);
            this.buffer.writeInt16BE(val, this.cursor);
            this.cursor += 2;
            return this;
        }
        else {
            const val = this.buffer.readInt16BE(this.cursor);
            this.cursor += 2;
            return val;
        }
    }
    int16(val, endian = "le") {
        return endian === "be" || val === "be" ? this.int16BE(val) : this.int16LE(val);
    }
    uint32LE(val) {
        if (typeof val === "number") {
            this.expand(4);
            this.buffer.writeUInt32LE(val < 0 ? val >>> 0 : val, this.cursor);
            this.cursor += 4;
            return this;
        }
        else {
            const val = this.buffer.readUInt32LE(this.cursor);
            this.cursor += 4;
            return val;
        }
    }
    uint32BE(val) {
        if (typeof val === "number") {
            this.expand(4);
            this.buffer.writeUInt32BE(val < 0 ? val >>> 0 : val, this.cursor);
            this.cursor += 4;
            return this;
        }
        else {
            const val = this.buffer.readUInt32BE(this.cursor);
            this.cursor += 4;
            return val;
        }
    }
    uint32(val, endian = "le") {
        return endian === "be" || val === "be" ? this.uint32BE(val) : this.uint32LE(val);
    }
    int32LE(val) {
        if (typeof val === "number") {
            this.expand(4);
            this.buffer.writeInt32LE(val, this.cursor);
            this.cursor += 4;
            return this;
        }
        else {
            const val = this.buffer.readInt32LE(this.cursor);
            this.cursor += 4;
            return val;
        }
    }
    int32BE(val) {
        if (typeof val === "number") {
            this.expand(4);
            this.buffer.writeInt32BE(val, this.cursor);
            this.cursor += 4;
            return this;
        }
        else {
            const val = this.buffer.readInt32BE(this.cursor);
            this.cursor += 4;
            return val;
        }
    }
    int32(val, endian = "le") {
        return endian === "be" || val === "be" ? this.int32BE(val) : this.int32LE(val);
    }
    floatLE(val) {
        if (typeof val === "number") {
            this.expand(4);
            this.buffer.writeFloatLE(val, this.cursor);
            this.cursor += 4;
            return this;
        }
        else {
            const val = this.buffer.readFloatLE(this.cursor);
            this.cursor += 4;
            return val;
        }
    }
    floatBE(val) {
        if (typeof val === "number") {
            this.expand(4);
            this.buffer.writeFloatBE(val, this.cursor);
            this.cursor += 4;
            return this;
        }
        else {
            const val = this.buffer.readFloatBE(this.cursor);
            this.cursor += 4;
            return val;
        }
    }
    float(val, endian = "le") {
        return endian === "be" || val === "be" ? this.floatBE(val) : this.floatLE(val);
    }
    bytes(bytes) {
        if (Array.isArray(bytes)) {
            this.expand(bytes.length);
            for (let i = 0; i < bytes.length; i++) {
                this.byte(bytes[i]);
            }
            return this;
        }
        else {
            const val = [];
            for (let i = 0; i < bytes; i++) {
                val.push(this.byte());
            }
            return val;
        }
    }
    buf(buff) {
        if (typeof buff !== "number") {
            const buffer = Buffer.isBuffer(buff) ? buff : buff.buffer;
            this.bytes([...buffer]);
            return this;
        }
        else {
            const buf = new DynamicBuffer(this.buffer.slice(this.cursor, this.cursor + buff));
            this.cursor += buff;
            return buf;
        }
    }
    string(str, encoding) {
        if (typeof str === "string") {
            this.upacked(str.length);
            this.expand(str.length);
            this.buffer.write(str, this.cursor, encoding || "utf8");
            this.cursor += str.length;
            return this;
        }
        else {
            const length = this.upacked();
            let str = "";
            for (let i = 0; i < length; i++) {
                str += this.char();
            }
            return str;
        }
    }
    packed(val) {
        if (typeof val === "number") {
            this.expand(Math.ceil(val.toString(16).length / 7));
            do {
                let b = val & 0b11111111;
                if (val >= 0b10000000) {
                    b |= 0b10000000;
                }
                this.uint8(b);
                val >>= 7;
            } while (val > 0);
            return this;
        }
        else {
            let out = 0;
            for (let shift = 0;; shift++) {
                const byte = this.uint8();
                const read = (byte >> 7) & 1;
                const val = read ? byte ^ 0b10000000 : byte;
                if (val) {
                    out |= val << shift * 7;
                }
                else if (read) {
                    out <<= shift * 7;
                }
                if (!read) {
                    break;
                }
            }
            return out;
        }
    }
    upacked(val) {
        if (typeof val === "number") {
            return this.packed(val >>> 0);
        }
        else {
            return this.packed() >>> 0;
        }
    }
    ip(val) {
        if (typeof val === "string") {
            return this.bytes(val.split(".").map(byte => parseInt(byte)));
        }
        else {
            return this.bytes(4).join(".");
        }
    }
}
exports.DynamicBuffer = DynamicBuffer;
